import AlteryxPythonSDK as Sdk
from abc import abstractmethod
from typing import Union, Callable, IO


class FileInputReader:
    def __init__(self,
                 stream: IO,
                 output_anchor: Sdk.OutputAnchor,
                 alteryx_engine: Sdk.AlteryxEngine,
                 stream_length_bytes: Union[int, None] = None,
                 field_source_resolver: Callable[[Sdk.Field], str] = lambda f: ''):
        self.record_info = None
        self.is_initialized = False
        self.record_creator = None
        self.field_source_resolver = field_source_resolver
        self.alteryx_engine = alteryx_engine
        self.output_anchor = output_anchor
        self.stream_length_bytes = stream_length_bytes
        self.stream = stream

        if self.stream_length_bytes is None and self.stream.seekable():
            self.stream.seek(0, 2)  # 2 = file end
            self.stream_length_bytes = self.stream.tell()
            self.stream.seek(0, 0)

    @abstractmethod
    def init(self) -> Sdk.RecordInfo:
        pass

    @abstractmethod
    def read(self, n_record_limit=-1) -> int:
        pass

    def report_input_stream_progress(self):
        stream_progress = round(self.stream.tell() / self.stream_length_bytes, 2)
        if (stream_progress * 100) % 5 == 0:
            self.output_anchor.update_progress(stream_progress)


class FileWriter:

    @abstractmethod
    def init(self, record_info: Sdk.RecordInfo):
        pass

    @abstractmethod
    def open(self):
        pass

    @abstractmethod
    def write(self, record: Sdk.RecordRef) -> bool:
        pass

    @abstractmethod
    def close(self):
        pass

    @abstractmethod
    def is_open(self):
        pass


