import json
import AlteryxPythonSDK as Sdk

from oauthlib.oauth2.rfc6749.parameters import (parse_token_response,
                                                prepare_token_request,
                                                prepare_token_revocation_request)
from oauthlib.oauth2 import WebApplicationClient, BackendApplicationClient


class AzureActiveDirectoryWebApplicationClient(WebApplicationClient):

    def __init__(self, client_id, client_secret, **kwargs):
        super().__init__(client_id, **kwargs)
        self.client_secret = client_secret

    def parse_request_body_response(self, body, scope=None, **kwargs):
        self.token = parse_token_response(body=body, scope=scope)
        params = json.loads(body)
        if 'expires_on' in params:
            self.token['expires_at'] = params['expires_on']
        self._populate_attributes(self.token)
        return self.token

    def prepare_refresh_body(self, body='', refresh_token=None, scope=None, **kwargs):
        refresh_token = refresh_token or self.refresh_token
        return prepare_token_request('refresh_token', body=body, scope=scope,
                                     refresh_token=refresh_token, client_id=self.client_id,
                                     client_secret=self.client_secret, **kwargs)


class AzureActiveDirectoryBackendApplicationClient(BackendApplicationClient):

    def __init__(self, client_id, client_secret, alteryx_engine: Sdk.AlteryxEngine=None, **kwargs):
        super().__init__(client_id, **kwargs)
        self.client_secret = client_secret
        self.alteryx_engine = alteryx_engine

    def prepare_request_uri(self, *args, **kwargs):
        if not self.alteryx_engine:
            raise NotImplementedError("Backend Application Clients do not make requests to the Authorization endpoints")
        else:
            raise NotImplementedError(self.alteryx_engine.xmsg("Backend Application Clients do not make requests to the Authorization endpoints"))

    def prepare_refresh_body(self, body='', refresh_token=None, scope=None, **kwargs):
        return self.prepare_request_body(
            body,
            scope,
            client_id=self.client_id,
            client_secret=self.client_secret,
            **kwargs
        )

